% By Martin M. Andreasen, April 2019
% This function computes the conditional moments up to k_period into the
% future for the control variable in the g-function.
% The computations are derived by using the Perturbation On Perturbation (POP) method.
% IMPORTANT: 1) This is for a "level" approximation under perfect foresight
%            2) The first time period in px, pxx, pxxx is the current time
%            period, i.e. we reproduce gx, gxx, gxxx, if firstOn = 1.
%
function [px,pxx,pxxx,p4x] = CondMoments_4th_levelCE_v3(gx,gxx,gxxx,g4x,hx,hxx,hxxx,h4x,k_period,firstOn,order_app)

% Allocating memory
ny    = size(gx,1);
nx    = size(hx,1);
if firstOn == 1
    k_period = k_period + 1;
end
px      = zeros(ny,k_period,nx);
pxx     = zeros(ny,k_period,nx,nx);
pxxx    = zeros(ny,k_period,nx,nx,nx);
p4x     = zeros(ny,k_period,nx,nx,nx,nx);

% The first time period
if firstOn == 1
    px(:,1,:)          = gx;
    pxx(:,1,:,:)       = gxx;
    pxxx(:,1,:,:,:)    = gxxx;
    p4x(:,1,:,:,:,:)   = g4x;
    startIndex         = 2;
else
    startIndex     = 1;
end

rx   = gx;
rxx  = reshape(gxx,ny,nx^2);
rxxx = reshape(gxxx,ny,nx^3);
r4x  = reshape(g4x,ny,nx^4);

for j=startIndex:k_period
    % ************** The first order effects *****************
    px(:,j,:) = rx*hx;
    
    %*************** The second order effects ***************
    if order_app > 1
        for alfa1=1:nx
            for alfa2=alfa1:nx
                %pxx(:,j,alfa1,alfa2) = hx(:,alfa1)'*rxx*hx(:,alfa2) + rx(1,:)*squeeze(hxx(:,alfa1,alfa2));
                pxx(:,j,alfa1,alfa2) = rxx*kron(hx(:,alfa1),hx(:,alfa2)) + rx*squeeze(hxx(:,alfa1,alfa2));
            end
            if alfa1 > 1
                pxx(:,j,alfa1,1:alfa1-1) = pxx(:,j,1:alfa1-1,alfa1);
            end
        end
    end
    % ***************** The third order effects *****************
    if order_app > 2
        for alfa1=1:nx
            for alfa2=alfa1:nx
                for alfa3=alfa2:nx
                    tmp = rxxx*kron(hx(:,alfa1),kron(hx(:,alfa2),hx(:,alfa3)));
                    pxxx(:,j,alfa1,alfa2,alfa3) = tmp...
                        +rxx*(kron(hx(:,alfa1),reshape(hxx(:,alfa2,alfa3),nx,1))...
                             +kron(reshape(hxx(:,alfa1,alfa3),nx,1),hx(:,alfa2))...
                             +kron(reshape(hxx(:,alfa1,alfa2),nx,1),hx(:,alfa3)))...
                        +rx*reshape(hxxx(:,alfa1,alfa2,alfa3),nx,1);
                    
                    % Using symmetry for alfa1 and alfa2
                    if alfa1 == alfa2 && alfa2 ~= alfa3 %alfa1==alfa2~=alfa3
                        %pxxx(:,j,alfa1,alfa1,alfa3)= pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa1,alfa3,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa3,alfa1,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3);
                    end
                    % Using symmetry for alfa2 and alfa3
                    if alfa1 ~= alfa2 && alfa2 == alfa3  %alfa1~=alfa2==alfa3
                        %pxxx(:,j,alfa1,alfa2,alfa2)= pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa2,alfa1,alfa2) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa2,alfa2,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3);
                    end
                    % Using symmetry for alfa1,alfa2, and alfa3
                    if alfa1 ~= alfa2 && alfa1 ~= alfa3 &&  alfa2 ~= alfa3 %alfa1~=alfa2~=alfa3
                        %pxxx(:,j,alfa1,alfa2,alfa3) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa1,alfa3,alfa2) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa3,alfa1,alfa2) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa3,alfa2,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa2,alfa3,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3);
                        pxxx(:,j,alfa2,alfa1,alfa3) = pxxx(:,j,alfa1,alfa2,alfa3);
                    end
                end
            end
        end
    end
    
    % ***************** Fourth order effects *****************
    if order_app > 3
        for alfa1=1:nx
            for alfa2=alfa1:nx
                for alfa3=alfa2:nx
                    for alfa4=alfa3:nx
                        tmp = r4x*kron(hx(:,alfa1),kron(hx(:,alfa2),kron(hx(:,alfa3),hx(:,alfa4))));
                        p4x(:,j,alfa1,alfa2,alfa3,alfa4) =  tmp + ...                                                   %1)
                            rxxx*(kron(hx(:,alfa1),kron(hx(:,alfa2),squeeze(hxx(:,alfa3,alfa4)))) + ...  %2)
                                  kron(hx(:,alfa1),kron(squeeze(hxx(:,alfa2,alfa4)),hx(:,alfa3))) + ...  %3)
                                  kron(squeeze(hxx(:,alfa1,alfa4)),kron(hx(:,alfa2),hx(:,alfa3))) + ...  %4)
                                  kron(hx(:,alfa1),kron(squeeze(hxx(:,alfa2,alfa3)),hx(:,alfa4))) )+ ... %5)
                            rxx*(kron(hx(:,alfa1),squeeze(hxxx(:,alfa2,alfa3,alfa4))) +...               %6)
                                 kron(squeeze(hxx(:,alfa1,alfa4)),squeeze(hxx(:,alfa2,alfa3))) )+...     %7)
                            rxxx*kron(squeeze(hxx(:,alfa1,alfa3)),kron(hx(:,alfa2),hx(:,alfa4)))  + ...  %8)
                            rxx*(kron(squeeze(hxx(:,alfa1,alfa3)),squeeze(hxx(:,alfa2,alfa4))) + ...     %9)
                                 kron(squeeze(hxxx(:,alfa1,alfa3,alfa4)),hx(:,alfa2))) +...              %10)
                            rxxx*kron(squeeze(hxx(:,alfa1,alfa2)),kron(hx(:,alfa3),hx(:,alfa4)))  + ...  %11)
                            rxx*(kron(squeeze(hxx(:,alfa1,alfa2)),squeeze(hxx(:,alfa3,alfa4))) + ...     %12)
                                 kron(squeeze(hxxx(:,alfa1,alfa2,alfa4)),hx(:,alfa3)) +...               %13)
                                 kron(squeeze(hxxx(:,alfa1,alfa2,alfa3)),hx(:,alfa4)))+...               %14)
                            rx*squeeze(h4x(:,alfa1,alfa2,alfa3,alfa4));                                  %15)
                        
                        
                        %1) Three identical and one different, for instance(1,1,1,2)
                        if alfa1 == alfa2 && alfa1 == alfa3 && alfa1~= alfa4
                            %p4x(:,j,alfa1,alfa2,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            p4x(:,j,alfa1,alfa1,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa4,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa4,alfa1,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                        end
                        
                        %2) Three identical and one different, for instance(1,2,2,2)
                        if alfa1 ~= alfa2 && alfa2 == alfa3 && alfa3 == alfa4
                            %p4x(:,j,alfa1,alfa2,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa1,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa2,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa2,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                        end
                        %3) Two identical and two different, for instance(1,1,2,2)
                        if alfa1 == alfa2 && alfa3 == alfa4 && alfa1 ~= alfa3
                            %p4x(:,j,alfa1,alfa1,alfa3,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa3,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa3,alfa1,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa3,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa3,alfa1,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa3,alfa3,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                        end
                        %4) To identical but three different, for instance (1,2,3,3)
                        if alfa1 ~= alfa2 && alfa3 == alfa4 && alfa1~= alfa3 && alfa2~=alfa3
                            %p4x(:,j,alfa1,alfa2,alfa3,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa1,alfa3,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa3,alfa3,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa3,alfa3,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa1,alfa3,alfa3,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa3,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa3,alfa2,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa3,alfa1,alfa3,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa3,alfa2,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa3,alfa1,alfa2,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa1,alfa3,alfa2,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa3,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                        end
                        %5) Two identical but three different, for instance (1,1,2,3)
                        if alfa1 == alfa2 && alfa3~= alfa4 && alfa1~=alfa3 && alfa2 ~= alfa3
                            %p4x(:,j,alfa1,alfa1,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa1,alfa4,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa3,alfa4,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa4,alfa3,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa3,alfa1,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa4,alfa1,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa1,alfa3,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa4,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa1,alfa3,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa4,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa3,alfa1,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa4,alfa1,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                        end
                        %6) Two identical but three different, for instance (1,2,2,3)
                        if alfa2 == alfa3 && alfa1 ~= alfa4 && alfa1 ~= alfa2 && alfa3 ~= alfa4
                            %p4x(:,j,alfa1,alfa2,alfa2,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa4,alfa2,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa2,alfa4,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa1,alfa2,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa2,alfa4,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa1,alfa4,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa4,alfa2,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa2,alfa4,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa4,alfa1,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa1,alfa4,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            
                            p4x(:,j,alfa2,alfa2,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            p4x(:,j,alfa2,alfa2,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                        end
                        %7) all different
                        if alfa1 ~= alfa2 && alfa1 ~= alfa3 && alfa1 ~= alfa4 && alfa2 ~= alfa3 && alfa2 ~= alfa4 && alfa3 ~= alfa4
                            %p4x(:,j,alfa1,alfa2,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            % Note this implementation implies that we
                            % "reproduce p4x(i,j,alfa1,alfa2,alfa3,alfa4) = p4x(i,j,alfa1,alfa2,alfa3,alfa4)"
                            
                            idx = perms([alfa1,alfa2,alfa3,alfa4]);
                            for ii=1:24
                                k1 = idx(ii,1);
                                k2 = idx(ii,2);
                                k3 = idx(ii,3);
                                k4 = idx(ii,4);
                                p4x(:,j,k1,k2,k3,k4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4);
                            end
                        end
                    end
                end
            end
        end
    end
    % Updating rx
    rx   = reshape(px(:,j,:),ny,nx);
    rxx  = reshape(pxx(:,j,:,:),ny,nx^2);
    rxxx = reshape(pxxx(:,j,:,:,:),ny,nx^3);
    r4x  = reshape(p4x(:,j,:,:,:,:),ny,nx^4);
end

% We reexpress the output if k_period = 1
if k_period == 1 && firstOn == 0
    px      = reshape(px(1:ny,k_period,1:nx),ny,nx);
    pxx     = reshape(pxx(1:ny,k_period,1:nx,1:nx),ny,nx,nx);
    pxxx    = reshape(pxxx(1:ny,k_period,1:nx,1:nx,1:nx),ny,nx,nx,nx);
    p4x     = reshape(p4x(1:ny,k_period,1:nx,1:nx,1:nx,1:nx),ny,nx,nx,nx,nx);
end