function [f,x,xp,y,yp,symparams,Phi] = RBCmodel(unitFree)
% This function reports the equations for the RBC model with variable labor
% no deterministic trend and internal habit formation (see King and Rebelo (1999)). 
% The model is reported in the vector function f, which must contain in the
% following order:
% i)   standard model equations
% ii)  links for lagged control variables appearing as states
% iii) linear exogenous shocks
%
% The function also reports the state vector (x and xp) and the control vector (y and yp)
% The ordering of these variables is as follows:
% x = [endogenous states, controls appearing in x, linear exogenous shocks]
% y = [controls appearing in x, controls not appearing in x]
% Note that the variables "controls appearing in x" must appear in the same
% order in x and y.
%
% Finally, for the number of these variables we use the following notation
% mx        = the number of endogenous state
% myx       = the number of controls appearing in x
% nx-mx-myx = the number of exogenous linear shocks
% nx        = number of elements in x
% ny        = number of elements in y
%
% By Martin Andreasen, April-2015

%% Section 1: Declaring coefficients and variables
%Define the structureal parameters
syms DELTA BETTA GAMA PHI ALFA RHOA
symparams=[DELTA,BETTA,GAMA,PHI,ALFA,RHOA];

%Define the state variables in this period (_cu) and the next period
%(_cup). Variables lagged by one period appear as "_ba1"
syms k1_cu  k2_cu  k3_cu  k4_cu  a1_cu  a2_cu  a3_cu  a4_cu
syms k1_cup k2_cup k3_cup k4_cup a1_cup a2_cup a3_cup a4_cup

%Define the control variables in this period (_cu) and the next period (_cup)
syms c1_cu  c2_cu  c3_cu  c4_cu  iv1_cu  iv2_cu  iv3_cu  iv4_cu  la_cu  
syms c1_cup c2_cup c3_cup c4_cup iv1_cup iv2_cup iv3_cup iv4_cup la_cup 

%% Section 2: Endogenous model equations 
A = (1-BETTA)/(ALFA*BETTA);
TAU = A^(1/GAMA);

% Eq 1: FOC for c1_cu 
if unitFree == 1
    eq1 = -1     + TAU*c1_cu^(-1/GAMA)/la_cu;
else
    eq1 = -la_cu + TAU*c1_cu^(-1/GAMA);
end

% Eq 2: FOC for c2_cu
if unitFree == 1
    eq2 = -1     + TAU*c2_cu^(-1/GAMA)/la_cu;
else
    eq2 = -la_cu + TAU*c2_cu^(-1/GAMA);
end

% Eq 3: FOC for c3_cu
if unitFree == 1
    eq3 = -1     + TAU*c3_cu^(-1/GAMA)/la_cu;
else
    eq3 = -la_cu + TAU*c3_cu^(-1/GAMA);
end

% Eq 4: FOC for c4_cu
if unitFree == 1
    eq4 = -1     + TAU*c4_cu^(-1/GAMA)/la_cu;
else
    eq4 = -la_cu + TAU*c4_cu^(-1/GAMA);
end

% Laws of motion for capital: k1_cu
if unitFree == 1
    eq5 = -1      + ((1-DELTA)*k1_cu + iv1_cu)/k1_cup;
else
    eq5 = -k1_cup + (1-DELTA)*k1_cu + iv1_cu;
end

% Laws of motion for capital: k2_cu
if unitFree == 1
    eq6 = -1      + ((1-DELTA)*k2_cu + iv2_cu)/k2_cup;
else
    eq6 = -k2_cup + (1-DELTA)*k2_cu + iv2_cu;
end

% Laws of motion for capital: k3_cu
if unitFree == 1
    eq7 = -1      + ((1-DELTA)*k3_cu + iv3_cu)/k3_cup;
else
    eq7 = -k3_cup + (1-DELTA)*k3_cu + iv3_cu;
end

% Laws of motion for capital: k4_cu
if unitFree == 1
    eq8 = -1      + ((1-DELTA)*k4_cu + iv4_cu)/k4_cup;
else
    eq8 = -k4_cup + (1-DELTA)*k4_cu + iv4_cu;
end


% FOC for capital: k1_cu
if unitFree == 1
    eq9 = (-la_cu*(1+PHI*(iv1_cu/k1_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a1_cup*A*k1_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv1_cup/k1_cup ...
        - 0.5*(iv1_cup/k1_cup-DELTA))*(iv1_cup/k1_cup-DELTA)))/(la_cu*(1+PHI*(iv1_cu/k1_cu-DELTA)));
else
    eq9 = -la_cu*(1+PHI*(iv1_cu/k1_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a1_cup*A*k1_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv1_cup/k1_cup ...
        - 0.5*(iv1_cup/k1_cup-DELTA))*(iv1_cup/k1_cup-DELTA));
end

% FOC for capital: k2_cu
if unitFree == 1
    eq10 = (-la_cu*(1+PHI*(iv2_cu/k2_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a2_cup*A*k2_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv2_cup/k2_cup ...
        - 0.5*(iv2_cup/k2_cup-DELTA))*(iv2_cup/k2_cup-DELTA)))/(la_cu*(1+PHI*(iv2_cu/k2_cu-DELTA)));
else
    eq10 = -la_cu*(1+PHI*(iv2_cu/k2_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a2_cup*A*k2_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv2_cup/k2_cup ...
        - 0.5*(iv2_cup/k2_cup-DELTA))*(iv2_cup/k2_cup-DELTA));
end

% FOC for capital: k3_cu
if unitFree == 1
    eq11 = (-la_cu*(1+PHI*(iv3_cu/k3_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a3_cup*A*k3_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv3_cup/k3_cup ...
        - 0.5*(iv3_cup/k3_cup-DELTA))*(iv3_cup/k3_cup-DELTA)))/(la_cu*(1+PHI*(iv3_cu/k3_cu-DELTA)));
else
    eq11 = -la_cu*(1+PHI*(iv3_cu/k3_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a3_cup*A*k3_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv3_cup/k3_cup ...
        - 0.5*(iv3_cup/k3_cup-DELTA))*(iv3_cup/k3_cup-DELTA));
end

% FOC for capital: k4_cu
if unitFree == 1
    eq12 = (-la_cu*(1+PHI*(iv4_cu/k4_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a4_cup*A*k4_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv4_cup/k4_cup ...
        - 0.5*(iv4_cup/k4_cup-DELTA))*(iv4_cup/k4_cup-DELTA)))/(la_cu*(1+PHI*(iv4_cu/k4_cu-DELTA)));
else
    eq12 = -la_cu*(1+PHI*(iv4_cu/k4_cu-DELTA)) ...
        + BETTA*la_cup*(1+ALFA*a4_cup*A*k4_cup^(ALFA-1) ...
        + PHI*((1-DELTA)+iv4_cup/k4_cup ...
        - 0.5*(iv4_cup/k4_cup-DELTA))*(iv4_cup/k4_cup-DELTA));
end


% Agg world resource constraint
lhs   = c1_cu + iv1_cu - DELTA*k1_cu ...
      + c2_cu + iv2_cu - DELTA*k2_cu ...
      + c3_cu + iv3_cu - DELTA*k3_cu ...
      + c4_cu + iv4_cu - DELTA*k4_cu;
rhs   = a1_cu*A*k1_cu^ALFA - PHI/2*k1_cu*(iv1_cu/k1_cu-DELTA)^2 ...
      + a2_cu*A*k2_cu^ALFA - PHI/2*k2_cu*(iv2_cu/k2_cu-DELTA)^2 ...
      + a3_cu*A*k3_cu^ALFA - PHI/2*k3_cu*(iv3_cu/k3_cu-DELTA)^2 ...
      + a4_cu*A*k4_cu^ALFA - PHI/2*k4_cu*(iv4_cu/k4_cu-DELTA)^2;
if unitFree == 1
    eq13 = -1 + rhs/lhs;
else
    eq13 = -lhs + rhs;
end

%% Section 3: Link equations

%% Section 4: The exogenous shocks
%Law of motion for technology: a1_cu
eq14 = -log(a1_cup) + RHOA*log(a1_cu);

%Law of motion for technology: a2_cu
eq15 = -log(a2_cup) + RHOA*log(a2_cu);

%Law of motion for technology: a3_cu
eq16 = -log(a3_cup) + RHOA*log(a3_cu);

%Law of motion for technology: a4_cu
eq17 = -log(a4_cup) + RHOA*log(a4_cu);

%Create function f
f = [eq1 ;eq2 ;eq3 ;eq4 ;eq5 ;eq6 ;eq7 ;eq8 ;eq9 ;eq10;...
     eq11;eq12;eq13;eq14;eq15;eq16;eq17];

%% Section 5: Defining x,xp,y,yp
% Define the vector of states, x and xp
x  = [k1_cu  k2_cu  k3_cu  k4_cu  a1_cu  a2_cu  a3_cu  a4_cu];
xp = [k1_cup k2_cup k3_cup k4_cup a1_cup a2_cup a3_cup a4_cup];
% Define the vector of controls, y and yp
y =  [c1_cu  c2_cu  c3_cu  c4_cu  iv1_cu  iv2_cu  iv3_cu  iv4_cu  la_cu ];
yp = [c1_cup c2_cup c3_cup c4_cup iv1_cup iv2_cup iv3_cup iv4_cup la_cup ];

% For the log-approximation
f   = subs(f, [x,y,xp,yp], exp([x,y,xp,yp]));

% Phi: the expected value for the exogenous shocks, else use Phi = []
Phi = [RHOA*log(a1_cu);RHOA*log(a2_cu);RHOA*log(a3_cu);RHOA*log(a4_cu);];
% For the log-approximation
Phi = subs(Phi, [x,y,xp,yp], exp([x,y,xp,yp]));

