% This program computes the deterministic steady state given 
% the structural parameters saved in params
% By Martin Andreasen April - 2019
function [auxOut,errorMes] = RBCmodel_ss(params)

%% Section 1: 
% The size of the model
N     = 8;
ny    = 2*N+1;      %Number of control variables
nx    = 2*N;        %Number of state variables
ne    = N;      %Number of shocks
mx    = N;      %Number of endogenous state variables (must come first in x)
myx   = 0;      %Number of lagged control variables appearing as states. 
                %These lagged control variables must come first in y AND must
                %appear in x after the endogenous states. 

%Unfold the params vector
DELTA = params.DELTA;
BETTA = params.BETTA;
GAMA  = params.GAMA;
ALFA  = params.ALFA;
RHOA  = params.RHOA;
STDA  = params.STDA;

% The eta matrix: nx*ne
eta      = zeros(nx,ne);
for i=1:N
    eta(N+i,i) = STDA;
end
% The higher order moments
% M.M2 is the expected value of kron(eps,eps). 
% M.M3 is the expected value of kron(eps,eps,eps) and so on. 
% Here I assume that the shock is standard normal.
% M.M2=1; M.M3=0; M.M4=3; M.M5=0; 
% If the shocks are independent standard normal you can use the command:
momEps =gaussian_moments(ne);

%% Section 2: Solving for the steady state
% Initializing errorMes: 0 for no errros, else 1
errorMes = 0;

% The value of capital and technology
Kss = 1; 
Ass = 1;

A = (1-BETTA)/(ALFA*BETTA);
TAU = A^(1/GAMA);

% The value of investment and consumption
IVss = DELTA;
Css  = A;

% The lagrange multiplier
LA = TAU*Css^(-1/GAMA);

%% Section 3: Reporting the output for the perturbation approximation
% For level approximation: k_cu = K;
% For log transformation: k_cu = log(K);
% For logistic transformation: k_cu = -log(1/Vss-1)

%The level of the states
auxOut.Xss        = [Kss*ones(1,N) Ass*ones(1,N)]';                      

%The level of the controls
auxOut.Yss        = [Css*ones(1,N) IVss*ones(1,N) LA]'; 

auxOut.labelx     = [{'k1_t'},{'k2_t'},{'k3_t'},{'k4_t'},{'k5_t'},{'k6_t'},...
                     {'k7_t'},{'k8_t'}
                     {'a1_t'},{'a2_t'},{'a3_t'},{'a4_t'},{'a5_t'},{'a6_t'},...
                     {'a7_t'},{'a8_t'}];
auxOut.labely     = [{'c1_t'},{'c2_t'},{'c3_t'},{'c4_t'},{'c5_t'},{'c6_t'},...
                     {'c7_t'},{'c8_t'},...
                     {'iv1_t'},{'iv2_t'},{'iv3_t'},{'iv4_t'},{'iv5_t'},{'iv6_t'},...
                     {'iv7_t'},{'iv8_t'},{'la_t'}];

%1 for a log-transformation, 2 for logistic transformation, 0 for a level approx
auxOut.transformX = ones(1,nx);    
auxOut.transformY = ones(1,ny);    

% Saving params in auxOut. This accounts for endogenous parameters (i.e.
% computed at the steady state), which can be added to params.
auxOut.params = params;
%% non-model specific part of output
%The transformed levels of controls
auxOut.yssTrans   = zeros(ny,1);
for i=1:ny
    if auxOut.transformY(i) == 1
        % Log-approximation
        auxOut.yssTrans(i,1) = log(auxOut.Yss(i,1));
    elseif auxOut.transformY(i) == 2
        % Logistic transformation
         auxOut.yssTrans(i,1) = -log(1/auxOut.Yss(i,1)-1);
    elseif auxOut.transformY(i) == 0
        % No transformation    
        auxOut.yssTrans(i,1) = auxOut.Yss(i,1);
    end
end

%The transformed levels of states
auxOut.xssTrans   = zeros(nx,1);
for i=1:nx
    if auxOut.transformX(i) == 1
        % Log-approximation
        auxOut.xssTrans(i,1) = log(auxOut.Xss(i,1));
    elseif auxOut.transformX(i) == 2
        % Logistic transformation
         auxOut.xssTrans(i,1) = -log(1/auxOut.Xss(i,1)-1);
    elseif auxOut.transformX(i) == 0
        % No transformation    
        auxOut.xssTrans(i,1) = auxOut.Xss(i,1);
    end
end
auxOut.ny         = ny;
auxOut.nx         = nx;
auxOut.ne         = ne;
auxOut.mx         = mx;
auxOut.myx        = myx;
auxOut.eta        = eta;
auxOut.momEps     = momEps;


end