function [f,x,xp,y,yp,symparams,Phi] = NKmodel(unitFree)
% This function reports the equations for a small NK model.
% The model is reported in the vector function f, which must contain in the
% following order:
% i)   standard model equations
% ii)  links for lagged control variables appearing as states
% iii) linear exogenous shocks
%
% The function also reports the state vector (x and xp) and the control vector (y and yp)
% The ordering of these variables is as follows:
% x = [endogenous states, controls appearing in x, linear exogenous shocks]
% y = [controls appearing in x, controls not appearing in x]
% Note that the variables "controls appearing in x" must appear in the same
% order in x and y.
%
% Finally, for the number of these variables we use the following notation
% mx        = the number of endogenous state
% myx       = the number of controls appearing in x
% nx-mx-myx = the number of exogenous linear shocks
% nx        = number of elements in x
% ny        = number of elements in y
%
% By Martin Andreasen, Aug-2019

%%Define the structural parameters
syms 	BETTA PHIzero PHIone PHItwo KAPA DELTA THETA ETA ALFA  ...
        RHOR  BETTAPAI BETTAY RHOA RHOD ...
        IV K R PAI Y ;
symparams=[BETTA,PHIzero,PHIone,PHItwo,KAPA,DELTA,THETA,ETA,ALFA,  ...
        RHOR,BETTAPAI,BETTAY,RHOA,RHOD,IV,K,R,PAI,Y];

%Define the state variables in this period (_cu) and the next period (_cup)
syms k_cu  s_cu  r_ba1  a_cu  d_cu
syms k_cup s_cup r_ba1p a_cup d_cup

%Define the control variables in this period (_cu) and the next period (_cup)
syms r_cu  c_cu  iv_cu  pai_cu  xtwo_cu  y_cu  w_cu  mc_cu  ptil_cu  h_cu  rk_cu  q_cu
syms r_cup c_cup iv_cup pai_cup xtwo_cup y_cup w_cup mc_cup ptil_cup h_cup rk_cup q_cup

%% The static model equations
% FOC for consumption
la_cu    = d_cu *c_cu ^(-PHItwo);
la_cup   = d_cup*c_cup^(-PHItwo);

% Omitting technology shocks
a_cu  = 1;
a_cup = 1;
%% The intertemporal equations
% Eq 1: FOC for captal
if unitFree == 1
    f1 = -1 + BETTA*la_cup*(rk_cup + q_cup*(1-DELTA) - q_cup*KAPA/2*(iv_cup/k_cup - IV/K)^2 ...
        + q_cup*KAPA*(iv_cup/k_cup - IV/K)*iv_cup/k_cup)/(q_cu*la_cu);
else
    f1 = -q_cu*la_cu + BETTA*la_cup*(rk_cup + q_cup*(1-DELTA) - q_cup*KAPA/2*(iv_cup/k_cup - IV/K)^2 + q_cup*KAPA*(iv_cup/k_cup - IV/K)*iv_cup/k_cup);
end

% EQ 2: Euler equation for the one period bond
if unitFree == 1
    f2 = -1 + BETTA*(1+r_cu)*la_cup/(la_cu*pai_cup);
else
    f2 = -la_cu + BETTA*(1+r_cu)*la_cup/pai_cup;
end

% EQ 3: Law of motion for the calvo pricing
if unitFree == 1
    f3 = -1 + (y_cu*mc_cu*ptil_cu^(-ETA-1) + ALFA*BETTA*la_cup/la_cu*(ptil_cu/ptil_cup)^(-ETA-1)*(1/pai_cup)^(-ETA)*(ETA-1)*xtwo_cup/ETA)/((ETA-1)*xtwo_cu/ETA);
    %f3 = -1 + (y_cu*mc_cu*1/ptil_cu^(ETA+1) + ALFA*BETTA*la_cup/la_cu*(ptil_cup/ptil_cu)^(ETA+1)*(pai_cup)^(ETA)*(ETA-1)*xtwo_cup/ETA)/((ETA-1)*xtwo_cu/ETA);
else
    f3 = -(ETA-1)*xtwo_cu/ETA + y_cu*mc_cu*ptil_cu^(-ETA-1) + ALFA*BETTA*la_cup/la_cu*(ptil_cu/ptil_cup)^(-ETA-1)*(1/pai_cup)^(-ETA)*(ETA-1)*xtwo_cup/ETA;
end

% EQ 4: Law of motion for the calvo pricing
if unitFree == 1
    f4 = -1 + y_cu*ptil_cu^(-ETA)/xtwo_cu + ALFA*BETTA*la_cup/la_cu*(ptil_cu/ptil_cup)^(-ETA)*(1/pai_cup)^(1-ETA)*xtwo_cup/xtwo_cu;
    %f4 = -1 + y_cu*1/ptil_cu^(ETA)/xtwo_cu + ALFA*BETTA*la_cup/la_cu*(ptil_cup/ptil_cu)^(ETA)*(pai_cup)^(ETA-1)*xtwo_cup/xtwo_cu;
else
    f4 = -xtwo_cu + y_cu*ptil_cu^(-ETA) + ALFA*BETTA*la_cup/la_cu*(ptil_cu/ptil_cup)^(-ETA)*(1/pai_cup)^(1-ETA)*xtwo_cup;
end

% The budget constraint
if unitFree == 1
    f5 = -1  +c_cu/y_cu + iv_cu/y_cu;
else
    f5 = -y_cu + c_cu  + iv_cu;
end

% The production function 
if unitFree == 1
    f6 = -1 + (a_cu*k_cu^THETA*h_cu^(1-THETA))/(y_cu*s_cup);
else
    f6 = -y_cu*s_cup + a_cu*k_cu^THETA*h_cu^(1-THETA);
end

% Household's FOC for hours
if unitFree == 1
    f7 = -1 + d_cu*PHIzero*(1-h_cu )^(-PHIone)/(la_cu*w_cu);
else
    f7 = -la_cu*w_cu + d_cu*PHIzero*(1-h_cu )^(-PHIone);
end

% Firms' FOC for hours
if unitFree == 1
    f8 = -1 + (a_cu *(1-THETA)*(h_cu /k_cu )^(-THETA))*mc_cu/w_cu;
else
    f8 = -w_cu + (a_cu *(1-THETA)*(h_cu /k_cu )^(-THETA))*mc_cu;
end

% The short rate by the taylor rule
f9 = -log(r_cu/(R-1)) + RHOR*log(r_ba1/(R-1)) + (1-RHOR)/(R-1)*(BETTAPAI*log(pai_cu/PAI) + BETTAY*log(y_cu/Y));

% Expression for the optimal relative price
f10 = -1+ptil_cu^(1-ETA)*(1-ALFA) +ALFA*(1/pai_cu )^(1-ETA);

% Firms' FOC for capital
if unitFree == 1
    f11 = -1  + a_cu *mc_cu *THETA*(h_cu /k_cu )^(1-THETA)/rk_cu;
else
    f11 = -rk_cu  + a_cu *mc_cu *THETA*(h_cu /k_cu )^(1-THETA);
end

% Tobin's q
f12 = -1+q_cu*(1-KAPA*(iv_cu /k_cu  - IV/K));

% EQ 6: Law of motion for k_cu
if unitFree == 1
   f13 = -1 + (1-DELTA)*k_cu/k_cup + iv_cu/k_cup - KAPA/2*(iv_cu/k_cu - IV/K)^2*k_cu/k_cup;  
else
   f13 = -k_cup + (1-DELTA)*k_cu + iv_cu - KAPA/2*(iv_cu/k_cu - IV/K)^2*k_cu;
end

% EQ 7: Law of motion for s_cu
if unitFree == 1
   f14 = -1 + (1-ALFA)*ptil_cu^-ETA/s_cup + ALFA*pai_cu^ETA*s_cu/s_cup; 
else
   f14 = -s_cup + (1-ALFA)*ptil_cu^(-ETA) + ALFA*pai_cu^ETA*s_cu; 
end

%% EQ 5: The link for r_ba1
if unitFree == 1
    f15 = -1 + r_cu/r_ba1p;
else
    f15 = -r_ba1p + r_cu;
end

%Law of motion for technology
%f16 = -log(a_cup) + RHOA*log(a_cu);

%Law of motion for demand shock
f16 = -log(d_cup) + RHOD*log(d_cu);

%Create function f
f = [f1;f2;f3;f4;f5;f6;f7;f8;f9;f10;f11;f12;f13;f14;f15;f16];

%% Define the vector of states and controls
% Controls, y and yp
y =  [r_cu  c_cu  iv_cu  pai_cu  xtwo_cu  y_cu  w_cu  mc_cu  ptil_cu  h_cu  rk_cu  q_cu];
yp = [r_cup c_cup iv_cup pai_cup xtwo_cup y_cup w_cup mc_cup ptil_cup h_cup rk_cup q_cup];

% States, x and xp: For log transformation
x  = [k_cu  s_cu  r_ba1  d_cu ];
xp = [k_cup s_cup r_ba1p d_cup ];

% For the log-approximation
f   = subs(f, [x,y,xp,yp], exp([x,y,xp,yp]));

% Phi: the expected value for the exogenous shocks, else use Phi = []
Phi = [RHOD*log(d_cu)];
% For the log-approximation
Phi = subs(Phi, [x,y,xp,yp], exp([x,y,xp,yp]));


