% This program computes the deterministic steady state given 
% the structural parameters saved in params
% By Martin Andreasen April - 2019
function [auxOut,errorMes] = NKmodel_ss(params)

%% Section 1: 
% The size of the model
ny    = 12;      %Number of control variables
nx    = 4;      %Number of state variables
ne    = 1;      %Number of shocks
mx    = 2;      %Number of endogenous state variables (must come first in x)
myx   = 1;      %Number of lagged control variables appearing as states. 
                %These lagged control variables must come first in y AND must
                %appear in x after the endogenous states. 
errorMes = 0;
%% Unfold the params vector
BETTA   = params.BETTA;
H       = params.H;
PHIone  = params.PHIone;
PHItwo  = params.PHItwo;
KAPA    = params.KAPA;
DELTA   = params.DELTA;
THETA   = params.THETA;
ETA     = params.ETA;
ALFA    = params.ALFA;
RHOR    = params.RHOR;
BETTAPAI= params.BETTAPAI;
BETTAY  = params.BETTAY;
PAI     = params.PAI;
RHOA    = params.RHOA;
RHOD    = params.RHOD;
STDA    = params.STDA;
STDD    = params.STDD;

% The eta matrix: nx*ne
eta      = zeros(nx,ne);
eta(4,1) = STDD;

% The higher order moments
% M.M2 is the expected value of kron(eps,eps). 
% M.M3 is the expected value of kron(eps,eps,eps) and so on. 
% Here I assume that the shock is standard normal.
% M.M2=1; M.M3=0; M.M4=3; M.M5=0; 
% If the shocks are independent standard normal you can use the command:
momEps =gaussian_moments(ne);

%% Section 2: Solving for the steady state
% The value of technology
A             = 1; 

% The relative optimal price for firms
PTIL          = ((1-ALFA*(1/PAI)^(1-ETA))/(1-ALFA))^(1/(1-ETA));

% The state variable s for distortions between output and produktion
S             = ((1-ALFA)*PTIL^(-ETA))/(1-ALFA*PAI^(ETA));

% The 1-period interest rate
R 			  = PAI/BETTA;

% The market price of capital
Q             = 1;          

% The real price of renting capital
RK            = Q*(1/BETTA-(1-DELTA)); 

% The marginal costs in the firms
MC            = (1-ALFA*BETTA*PAI^(ETA))*(ETA-1)/ETA*PTIL/(1-ALFA*BETTA*PAI^(-(1-ETA)));

% The capital stock
K             = H*(RK/(MC*THETA))^(1/(THETA-1));

% The wage level 
W             = MC*(1-THETA)*(K/H)^THETA;                                     

% The level of investment
IV            = K - (1-DELTA)*K;

% The consumption level
C              = (K^THETA*H^(1-THETA)/S-IV);

% The output level 
Y             = C + IV;

% The value of lampda  
LA            = (C)^-PHItwo;

% The value of PHIzero
PHIzero       = LA*W*(1-H)^PHIone;                                            

% The value of X2
X2            = Y*PTIL^(-ETA)/(1-BETTA*ALFA*PAI^(-(1-ETA)));                                         

%% Section 3: Reporting the output for the perturbation approximation
% For level approximation: k_cu = K;
% For log transformation: k_cu = log(K);
% For logistic transformation: k_cu = -log(1/Vss-1)

%The level of the states
% k_cu  s_cu  r_ba1  d_cu
auxOut.Xss        = [K S R-1 1]';                      

%The level of the controls
% [r_cu  c_cu  iv_cu  pai_cu  xtwo_cu  y_cu  w_cu  mc_cu  ptil_cu  h_cu rk_cu q_cu];
auxOut.Yss        = [R-1 C IV PAI X2 Y W MC PTIL H RK Q]'; 

auxOut.labelx     = [{'k_{t}'},{'s_t'},{'r_{t-1}'},{'d_t'}];
auxOut.labely     = [{'r_t'},{'c_t'},{'iv_t'},{'pai_t'},{'x2_t'},...
                     {'y_t'},{'w_t'},{'mc_t'},{'ptil_t'},{'h_t'},{'rk_t'},{'q_t'}];

%1 for a log-transformation, 2 for logistic transformation, 0 for a level approx
%auxOut.transformX = ones(1,nx);    
auxOut.transformX = [1 1 1 1];    
auxOut.transformY = ones(1,ny);    

% Saving params in auxOut. This accounts for endogenous parameters (i.e.
% computed at the steady state), which can be added to params.
params.PHIzero = PHIzero;
params.IV      = IV;
params.K       = K;
params.R       = R;
params.Y       = Y;
auxOut.params = params;
%% non-model specific part of output
%The transformed levels of controls
auxOut.yssTrans   = zeros(ny,1);
for i=1:ny
    if auxOut.transformY(i) == 1
        % Log-approximation
        auxOut.yssTrans(i,1) = log(auxOut.Yss(i,1));
    elseif auxOut.transformY(i) == 2
        % Logistic transformation
         auxOut.yssTrans(i,1) = -log(1/auxOut.Yss(i,1)-1);
    elseif auxOut.transformY(i) == 0
        % No transformation    
        auxOut.yssTrans(i,1) = auxOut.Yss(i,1);
    end
end

%The transformed levels of states
auxOut.xssTrans   = zeros(nx,1);
for i=1:nx
    if auxOut.transformX(i) == 1
        % Log-approximation
        auxOut.xssTrans(i,1) = log(auxOut.Xss(i,1));
    elseif auxOut.transformX(i) == 2
        % Logistic transformation
        auxOut.xssTrans(i,1) = -log(1/auxOut.Xss(i,1)-1);
    elseif auxOut.transformX(i) == 0
        % No transformation    
        auxOut.xssTrans(i,1) = auxOut.Xss(i,1);
    end
end
auxOut.ny         = ny;
auxOut.nx         = nx;
auxOut.ne         = ne;
auxOut.mx         = mx;
auxOut.myx        = myx;
auxOut.eta        = eta;
auxOut.momEps     = momEps;


end