%Martin M. Andreasen, Jan 2019
addpath(genpath(pwd))
clear
close all
warning off
%clc
%% Settings
GH_points       = 5;       %Number of points for numerical integration
xPointsDim      = 0;       %Number of points per state dimension for Accuracy grid. 
                           % If 0, then use sim path

orderApp        = 2;       %Order of approximation for the perturbation solution

% Configurations for the Extended Perturnation
setupEPer.Nmax            = 200;       %Max steps in the Extended Path
setupEPer.Nmin            = 50;        %Min steps in the Extended Path
setupEPer.maxDistSS       = 0.03;      %Max distance to SS when determining N
setupEPer.orderAppStart   = 1;         %Order of approximation for the starting values in the fixed point solver
setupEPer.fixedPointSolver= 1;         %1 for the Newton-Raphson solver,
                                       %2 Newton-Raphson solver with optimal delta,
                                       %3 for an LM algorithm that minimizes the residuals
                                       %4 for an LM algorithm that minimizes the weighted residuals
setupEPer.JacobianOption  = 3;         %1 for using numerical J and numerically solving the system J*x = -f
                                       %2 for using analytical J and numerically solving the system J*x = -f
                                       %3 for using analytical J and analytically solving the system J*x = -f
setupEPer.lambda0         = 1e-6;      %Tuining parameter for fixedPointSolver = 3
setupEPer.lambdaBackup    = 1e-2;      %Tuining parameter for the LM algorithm when used as backup
setupEPer.tolf            = 1e-6;      %Tolerance level for optimization problem
setupEPer.MaxIter         = 1D4;       %Maximum number of iterations allowed in the Extended Path algorithm
setupEPer.residualMax     = 1D-4;      %Max allowed value of a residual - for the hybrid simulator
setupEPer.MexOn           = 1;         %1 For using MEX-files, else 0
%% Model parameters
paramsVersion = 1; %jj;
[params,gridFactor_x] = getParamsVersion(paramsVersion);
gridFactor_x = gridFactor_x.*0+3;

%% The perturbation approximation
load([pwd,'\ModelSpecification\setupModel'],'setupModel')
[model,errorMes] = perturbationDSGEmodel(params,orderApp,setupModel,setupEPer.MexOn);

% The setup structure for extended perturbation
setupEPer = infoToSetupEPer(model,setupEPer);

% Standard deviations of the states in linearized model
nx   = size(model.eta,1);
varX = reshape((eye(nx^2)-kron(model.hx,model.hx))\reshape(model.eta*model.eta',nx^2,1),nx,nx);
stdX = sqrt(max(diag(varX),1D-6));

% Information for constructing the grid of x_cu
[setupEE.GH_e,setupEE.GH_w] = GaussHermite(GH_points);
setupEE.lowerx     = -gridFactor_x.*stdX;
setupEE.upperx     =  gridFactor_x.*stdX;
setupEE.xPointsDim = xPointsDim;
setupEE.setupEPer  = setupEPer;
% Special limits for s_cu
if paramsVersion == 1 
    setupEE.lowerx(2,1) = 0.000;
    setupEE.upperx(2,1) = 0.04;
elseif paramsVersion == 2
    setupEE.lowerx(2,1) = 0.000;
    setupEE.upperx(2,1) = 0.1;
end

%% Accuracy at the ZLB
numPlot = 50;
dimx  = 4;
posr  = find(strcmp(model.labely,'r_t'));
xPlot = zeros(model.nx,numPlot);
xPlot(1,:) = 2*stdX(1,1);
xPlot(3,:) = -2.5*stdX(3,1);
yEPer = zeros(model.ny+1,numPlot);
yPer  = zeros(model.ny+1,numPlot);
for i=1:numPlot
    i
    xPlot(dimx,i) = setupEE.lowerx(dimx,1)+i*(setupEE.upperx(dimx,1)-setupEE.lowerx(dimx,1))/numPlot;
    
    yPer(1:model.ny,i)=policy(xPlot(:,i)+model.h0,model.g0,model.h0,model.derivs,model.orderApp);
    yPer(model.ny+1,i) = exp(yPer(posr,i));
    
    yEPer(1:model.ny,i) = policyFunctionEPer(xPlot(:,i)+model.h0,model,setupEPer);
    yEPer(model.ny+1,i) = exp(yEPer(posr,i));
end

% Linearization solution for model with standard Taylor rule
yEPerSlope  = yEPer(:,numPlot/2)-yEPer(:,numPlot/2-1);
yLinear     = yEPer(:,numPlot/2)-yEPerSlope*(numPlot/2-1:-1:-numPlot/2);

% Accuracy
setupEE.xSim = xPlot ;
setupEE.xPointsDim = 0;
eulerErrorEPer = EulerEqError_mp(model,1,model.orderApp,setupEE);
eulerErrorPer  = EulerEqError(model,0,model.orderApp,setupEE);

%% Plotting the results
close all
xValues         = 4:40; 
figureSize      = [0 0 1800 600+300]; % (0,0) size in x size in y
figure('Name','Accuracy at ZLB','NumberTitle','off','Position',figureSize);
markerSize      = 5;
fontSizeValue   = 15;
fontSizeText    = 18;
LineWidthValue  = 3;

posrLevel = model.ny+1;
posInfl   = find(strcmp(model.labely,'pai_t'));
posc      = find(strcmp(model.labely,'c_t'));
posr      = find(strcmp(model.labely,'r_t'));

subplot(2,2,1)
hold on
plot(xPlot(dimx,xValues),4*yEPer(posrLevel,xValues),'-k','LineWidth',LineWidthValue)
plot(xPlot(dimx,xValues),4*yPer(posrLevel,xValues),'-*b')
plot(xPlot(dimx,xValues),4*yLinear(posrLevel,xValues),'--k')
set(gca,'FontSize',fontSizeValue) 
title('Annualized short rate: $r_t$','interpreter','Latex','FontSize',fontSizeText)
xlabel('$\hat{d}_t$','interpreter','Latex','FontSize',fontSizeText)
axis tight
legend({'Extended Per','Standard Per','Linear'},'Orientation','horizontal','Location','NorthEast','EdgeColor',[1 1 1],...
            'interpreter','Latex','FontSize',fontSizeText);

subplot(2,2,2)
hold on
plot(xPlot(dimx,xValues),yEPer(posInfl,xValues),'-k','LineWidth',LineWidthValue)
plot(xPlot(dimx,xValues),yPer(posInfl,xValues),'-*b')
plot(xPlot(dimx,xValues),yLinear(posInfl,xValues),'--k')
set(gca,'FontSize',fontSizeValue) 
title('Inflation rate: $\hat{\pi_t}$','interpreter','Latex','FontSize',fontSizeText)
xlabel('$\hat{d}_t$','interpreter','Latex','FontSize',fontSizeText)
axis tight

subplot(2,2,3)
hold on
plot(xPlot(dimx,xValues),yEPer(posc,xValues)-model.g0(posc,1),'-k','LineWidth',LineWidthValue)
plot(xPlot(dimx,xValues),yPer(posc,xValues)-model.g0(posc,1),'-*b')
plot(xPlot(dimx,xValues),yLinear(posc,xValues)-model.g0(posc,1),'--k')
set(gca,'FontSize',fontSizeValue) 
title('Consumption: $\hat{c}_t$','interpreter','Latex','FontSize',fontSizeText)
xlabel('$\hat{d}_t$','interpreter','Latex','FontSize',fontSizeText)
axis tight

subplot(2,2,4)
hold on
plot(xPlot(dimx,xValues),log10(mean(abs(eulerErrorEPer(xValues,:)),2)),'-k','LineWidth',LineWidthValue)
plot(xPlot(dimx,xValues),log10(mean(abs(eulerErrorPer(xValues,:)),2)),'-*b')
set(gca,'FontSize',fontSizeValue) 
title('Euler Errors: $\log_{10}$(MAE$_t$)','interpreter','Latex','FontSize',fontSizeText)
xlabel('$\hat{d}_t$','interpreter','Latex','FontSize',fontSizeText)
axis tight

